/*
 * Settings.cpp
 *
 *  Created on: 8 Oct 2020
 *      Author: Jack
 */

#include <Model/Settings.hpp>
#include "driverlib/eeprom.h"
#include <stdint.h>


void Settings::setInsertDist(int value)
{
    insertDist = value;
}

void Settings::setReverseDist(int value)
{
    reverseDist = value;
}

void Settings::setJogDist(int value)
{
    jogDist = value;
}

void Settings::setFullInsertionPause(int value)
{
    fullInsertionPause = value;
}

void Settings::setTrapezoidEnabled(bool value)
{
    enableTrapezoid = value;
}

void Settings::setInsertSpeed(int value)
{
    insertSpeed = value;
}

void Settings::setRampTime(int value)
{
    rampTime = value;
}

void Settings::setHighDwellTime(int value)
{
    highDwellTime = value;
}

void Settings::setLowDwellTime(int value)
{
    lowDwellTime = value;
}

void Settings::setDistPerTrapezoid(int value)
{
    distPerTrapezoid = value;
}

void Settings::startSelfCal()
{
    selfCalMode = true;
}

void Settings::endSelfCal()
{
    selfCalMode = false;
}

void Settings::setOutputSwitchPend(bool value)
{
    outputSwitchPend = value;
}
void Settings::setOutputSwitchTargetRaw(bool value)
{
    outputSwitchTargetRaw = value;
}

void Settings::setTorqueLimit(int value)
{
    torqueLimit = value;
}
void Settings::setReverseSpeed(int value)
{
    reverseSpeed = value;
}

void Settings::setInvert(bool value)
{
    invertSystem = value;
}



int Settings::getInsertDist()
{
    return insertDist;
}

int Settings::getReverseDist()
{
    return reverseDist;
}

int Settings::getJogDist()
{
    return jogDist;
}

int Settings::getFullInsertionPause()
{
    return fullInsertionPause;
}

bool Settings::getTrapezoidEnabled()
{
    return enableTrapezoid;
}

int Settings::getInsertSpeed()
{
    return insertSpeed;
}

int Settings::getRampTime()
{
    return rampTime;
}

int Settings::getHighDwellTime()
{
    return highDwellTime;
}

int Settings::getLowDwellTime()
{
    return lowDwellTime;
}

int Settings::getDistPerTrapezoid()
{
    return distPerTrapezoid;
}



void Settings::setEnableMotor(bool value)
{
    enableMotor = value;
}

bool Settings::getEnableMotor()
{
    return enableMotor;
}


int Settings::getSelfCalActive()
{
    return selfCalMode;
}

bool Settings::getOutputSwitchPend()
{
    return outputSwitchPend;
}
bool Settings::getOutputSwitchTargetRaw()
{
    return outputSwitchTargetRaw;
}
int Settings::getTorqueLimit()
{
    return torqueLimit;
}
int Settings::getReverseSpeed()
{
    return reverseSpeed;
}

bool Settings::getInvert()
{
    return invertSystem;
}



static void TestEEPROM()
{
    uint32_t eepromSize = EEPROMSizeGet();
    uint32_t settingsSize = sizeof(Settings);
    if(settingsSize > eepromSize)
    {
        //error eeprom size too small
        while(1){};
    }
}

bool Settings::loadEEPROM()
{
    TestEEPROM();
    Settings newSettings;
    EEPROMRead((uint32_t*)&newSettings, 0, sizeof(Settings));
    if(newSettings.MAGIC_NUMBER == EXPECTED_MAGIC_NUMBER && newSettings.SETTINGS_VERSION == EXPECTED_VERSION_NUMBER)
    {
        //found valid settings
        *this = newSettings;
        outputSwitchPend = false;
        outputSwitchTargetRaw = false;
        selfCalMode = false;
        enableMotor = false;
        return true;
    }
    else
    {
        return false;
    }
}


void Settings::saveEEPROM()
{
    TestEEPROM();
    EEPROMProgram((uint32_t*)this, 0, sizeof(Settings));

}
void Settings::resetEEPROM()
{
    TestEEPROM();
    Settings newSettings;
    EEPROMProgram((uint32_t*)&newSettings, 0, sizeof(Settings));
}
