/*
 * Settings.hpp
 *
 *  Created on: 8 Oct 2020
 *      Author: Jack
 */

#ifndef SETTINGS_HPP_
#define SETTINGS_HPP_

#include <stdint.h>
#define EXPECTED_MAGIC_NUMBER 0x26be63ff
#define EXPECTED_VERSION_NUMBER 1

alignas(8) class Settings
{
    uint32_t MAGIC_NUMBER = EXPECTED_MAGIC_NUMBER;
    uint32_t SETTINGS_VERSION = EXPECTED_VERSION_NUMBER;
    bool enableMotor = false; //enable motor driver
    uint32_t insertDist = 12*4000;
    uint32_t reverseDist = 14*4000;
    uint32_t jogDist = 1;
    uint32_t fullInsertionPause = 1000;
    bool enableTrapezoid = false;
    //for no trapezoid
    uint32_t insertSpeed = 2000;//CPS
    //for trapezoid
    uint32_t rampTime = 150;//ms to reach full speed
    uint32_t highDwellTime = 200;//ms at full speed
    uint32_t lowDwellTime = 250;//ms stopped
    uint32_t distPerTrapezoid = 2000;//distance of turning for each trapezoid
    bool outputSwitchPend = false;//whether waiting for task to complete output switch
    bool outputSwitchTargetRaw = false;//when waiting or completing output switch, new state should be raw
    bool selfCalMode = false;
    uint32_t torqueLimit = 1000; //fixed point, 2dp 1000 -> 10.00 Nm
    int32_t reverseSpeed = -1; //fixed point, 2dp 1000 -> 10.00 Nm : -1 = same as insertion
    bool invertSystem = false; //whether motor control, and measured values, should be reversed
public:
    void setEnableMotor(bool value);
    void setInsertDist(int value);
    void setReverseDist(int value);
    void setJogDist(int value);
    void setFullInsertionPause(int value);
    void setTrapezoidEnabled(bool value);
    void setInsertSpeed(int value);
    void setRampTime(int value);
    void setHighDwellTime(int value);
    void setLowDwellTime(int value);
    void setDistPerTrapezoid(int value);
    void setOutputSwitchPend(bool value);
    void setOutputSwitchTargetRaw(bool value);
    void startSelfCal();
    void endSelfCal();;
    void setTorqueLimit(int value);
    void setReverseSpeed(int value);
    void setInvert(bool value);

    bool getEnableMotor();
    int getInsertDist();
    int getReverseDist();
    int getJogDist();
    int getFullInsertionPause();
    bool getTrapezoidEnabled();
    int getInsertSpeed();
    int getRampTime();
    int getHighDwellTime();
    int getLowDwellTime();
    int getDistPerTrapezoid();
    int getSelfCalActive();
    bool getOutputSwitchPend();
    bool getOutputSwitchTargetRaw();
    int getTorqueLimit();
    int getReverseSpeed();
    bool getInvert();


    bool loadEEPROM();
    void saveEEPROM();
    void resetEEPROM();
};

#endif /* SETTINGS_HPP_ */
