/*
 * DatalogTask.hpp
 *
 *  Created on: 2 Sep 2020
 *      Author: Jack
 */

#ifndef TASKS_DATALOGTASK_HPP_
#define TASKS_DATALOGTASK_HPP_

#include <Model/Enums.hpp>
#include <Model/Settings.hpp>
#include <Tasks/BaseTask.hpp>
#include <Wrappers/Semaphore.hpp>
#include <Wrappers/Queue.hpp>
#include <Wrappers/QEI.hpp>
#include <Wrappers/UART.hpp>
#include <Model/Status.hpp>
#include "driverlib/udma.h"

#define CAL_TOTAL_COUNTS (360*4)
#define CAL_BINS (360/5)
#define CAL_BIN_SIZE (CAL_TOTAL_COUNTS/CAL_BINS)
#define CAL_MAX_SAMPLES 400

class DatalogTask: public BaseTask
{
    Status* status;
    QEI* displ_enc;
    QEI* sensor_enc;
    UART* uart;
    Settings* settings;
    static Semaphore gotReadingSemaphore;
    static void ISRHandler();
    uint16_t ADCbuffers[4][250];
    static tDMAControlTable DMATaskList[4];
    static unsigned char nextBuffer;
    int64_t calAccumulator[CAL_BINS] = {0};
    int16_t calCount[CAL_BINS] = {0};
    int32_t calTable[CAL_BINS] = {0};

    uint32_t filteredSample();
protected:
    void Setup();
    void TaskMethod();
public:
    DatalogTask(Status* status,
                QEI* motor_enc,
                QEI* sensor_enc,
                Settings* settings,
                UART* uart);
};

#endif /* TASKS_DATALOGTASK_HPP_ */
