/*
 * UartStatus.cpp
 *
 *  Created on: 11/08/2019
 *      Author: jwi123
 */

#include <Tasks/UartStatusTask.hpp>
#include "FreeRTOS.h"
#include "utils/ustdlib.h"

UartStatusTask::UartStatusTask(Status* status, UART* uart, Settings* settings)
    : uart(uart),
      status(status),
      settings(settings)
{
    priority = 5;
    stackSize = 256;
    taskName = "UartStatusTask";
}


void UartStatusTask::Setup()
{

}

void UartStatusTask::TaskMethod()
{
    uart->Send("   \n\r\033[2JUart output started...\n\r");

    TickType_t xLastWakeTime = xTaskGetTickCount();


    volatile UBaseType_t uxHighWaterMark;
    while(1)
    {
        uxHighWaterMark = uxTaskGetStackHighWaterMark( NULL );
        char string[60];

        vTaskDelayUntil(&xLastWakeTime, 1000);

        if(settings->getOutputSwitchTargetRaw() == true)
        {
            if(settings->getOutputSwitchPend())
            {
                settings->setOutputSwitchPend(false);
            }
            continue;
        }
        else if(!settings->getOutputSwitchPend())
        {
            int invert = settings->getInvert() ? -1 : 1;
            uart->Send("\n***Test Rig Status***\n\n\r");
            usnprintf (string, sizeof(string), "Disp. Encoder:  %d\033[K\n\r",
                       status->getDisplacmentEncoderCal());
            uart->Send(string);

            usnprintf (string, sizeof(string), "Motor Speed:    %d\033[K\n\r",
                       status->getMotorSpeed());
            uart->Send(string);

            usnprintf (string, sizeof(string), "Sensor Encoder: %d\033[K\n\r",
                       status->getSensorEncoderCal()*invert);
            uart->Send(string);

            int torqueFP3dp = status->getSensorTorque()*invert;
            int whole = abs(torqueFP3dp)/1000;
            int fraction = abs(torqueFP3dp)%1000;
            char sign = (torqueFP3dp > 0) ? '+' : '-';

            usnprintf (string, sizeof(string), "Sensor Torque:  %c%02d.%03d Nm\033[K\n\r",
                       sign, whole, fraction);
            uart->Send(string);

            usnprintf (string, sizeof(string), "?> %s\033[K\n\r", status->getInputString());
            uart->Send(string);


            usnprintf (string, sizeof(string), "%s\033[K\r\033[8A",status->getOutputString());
            uart->Send(string);
            /*
            int torqueFP3dp = status->getSensorTorque();
            int whole = torqueFP3dp/1000;
            int fraction = abs(torqueFP3dp)%1000;
            usnprintf (string, sizeof(string), "%02d.%03d\n\r",
                                          whole, fraction);
            uart->Send(string);
            */
        }


    }

}
